;; MyInt: This sort represents 8-bit bitvecs. We use 8 bits instead of the
;; normal 32 because the printed values are easier to read! All the same
;; general principals apply.
(define-sort MyInt () (_ BitVec 8))

;; These represent the parameters to our method.
(declare-const a MyInt)
(declare-const b MyInt)
(declare-const c MyInt)

;; Here we do some SSA style assignments to get the final value of trian. If you
;; don't know about SSA (Singal Static Assignment), don't worry! It's a fancy
;; way of saying that we turn code with changing variable values like:
;;
;;     int a = 1;
;;     a = a + 1;
;;     a = a + 1;
;;
;; into code without any side effects like:
;;
;;     int a1 = 1;
;;     int a2 = a1 + 1;
;;     int a3 = a2 + 1;
;;
;; Since the `trian` variable gets updated throughout the `classify` method,
;; we break it up into `trian`, `trian2`, and `trian3`, corresponding to the
;; amount added to `trian` on lines 25, 28, and 31 respectively.
(declare-const trian1 MyInt)
(declare-const trian2 MyInt)
(declare-const trian3 MyInt)

;; The final value of `trian` after line 32.
(declare-const trian  MyInt)

; Conditionals in if statements given by their line numbers in the initial
; source code. Thus line-20-cond corresponds to 
; 
;       if (a <= 0 || b <= 0 || c <= 0) { ...
;
; This is provided for convenience

(declare-const line-20-cond bool) ; if (a <= 0 || b <= 0 || c <= 0)
(declare-const line-24-cond bool) ; if (a == b)
(declare-const line-27-cond bool) ; if (a == c)
(declare-const line-30-cond bool) ; if (b == c)
(declare-const line-33-cond bool) ; if (trian == 0)

; Assign the values of the conditionals
(assert (=  line-20-cond (or  (bvsle a #x00)
                              (bvsle b #x00)
                              (bvsle c #x00))))

(assert (= trian1 (if_then_else line-24-cond #x01 #x00)))
(assert (= trian2 (if_then_else line-27-cond #x02 #x00)))
(assert (= trian3 (if_then_else line-30-cond #x03 #x00)))

(assert (= trian (bvadd trian1 trian2 trian3)))

(declare-const initial-condition bool)
(declare-const mutated-condition bool)

;;;;;;;;;;;;;;;;; START STUDENT CODE ;;;;;;;;;;;;;;;

;; The following is a diff between the original program and the mutant.
;;
;; # MUTANT 59
;; @@ -33 +33 @@
;; -        if (trian == 0) {
;; +        if (trian <= 0) {

;; TODO 1: When the condition on line 20 is `true` the method returns before the
;;    mutation on line 34 is reached. Therefore, we can conclude that
;;    line-20-cond is `false`. Write an assertion to enforce this.


;; TODO 2: Write assertions for the conditions on lines 24, 27, 30, and 33. Note
;;    that while most operators on BitVecs have special names (such as `bvadd`
;;    or `bvsgt`), `=` works everywhere...in some sense, equality is the only
;;    thing that means the same thing no matter what Sort of thing you are
;;    comparing! This means that I can assert that `Int`s are equal with
;;
;;        `(assert (= myInt 2))`,
;;
;;    and I can assert that `BitVec`s are equal with
;;
;;        `(assert (= myBV #x04))`,
;;
;;    both using the same `=` operator.


;; TODO 3: Write assertions to constrain `initial-condition` and
;; `mutated-condition`


;;;;;;;;;;;;;;;;; END STUDENT CODE ;;;;;;;;;;;;;;;

(assert (not (= mutated-condition initial-condition)))

(echo "Checking Satisfiability:")
(check-sat)
;; If needed, run
;; (echo "-------------------------------- Getting model ---------------------------------")
;; (get-model)
